#| theme.jl for t:o)m's new theme design

   Copyright (C) 2001 Thomas "t:o)m" Frank

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   $Id: theme.jl,v 0.73 2001/12/23 20:18:56 jsh Exp $

   changelog:
   
     v0.73 customization fix
     v0.71 first release of aBOX
     
   Authors: Thomas "t:o)m" Frank <tom@madguru.de>
|#
					    
(require 'rep.data.tables)		;need hash tables for icon cache
(require 'sawfish.wm.util.recolor-image)

(defgroup aBOX "aBOX Theme"
  :group appearance)

(defcustom aBOX:text-justify 'left
  "Text is \\w justified in window titles."
  :type (choice left right center)
  :group (appearance aBOX)
  :after-set after-setting-frame-option)

(define (text-justifier w)
  (case aBOX:text-justify
    ((left) 'left)
    ((right) 'right)
    ((center) 'center)))

;; images

(let*
;; Update window title pixel length
    ((title-width
      (lambda (w)
      (let ((w-width (car (window-dimensions w))))
	  (max 0 (min (- w-width 100) (text-width (window-name w))))))))

;; 21x24
(define top-left-border
  (list (make-image "i-top-border-left.png")
	(make-image "top-border-left.png")))

;; 21x18?
(define top-right-border
  (list (make-image "i-top-border-right.png")
	(make-image "top-border-right.png")))

;; 28x20
(define top-left
  (list (make-image "i-top-left.png")
	(make-image "top-left.png")))

;; 2xx4
(define top-left-sting
  (list (set-image-border
	 (make-image "i-top-teal-left.png") 0 13 0 0)
	(set-image-border
	 (make-image "top-teal-left.png") 0 13 0 0)))

;; 44x20
(define top-center-right-border
  (list (make-image "i-top-teal-left.png")
	(make-image "top-teal-left.png")))

;; 44x20
(define top-center-left
  (list (make-image "i-top-teal-left.png")
	(make-image "top-teal-left.png")))

;; 2x20
(define top-center-mid
  (list (make-image "i-top-middle.png")
	(make-image "top-middle.png")))

;; 2x20
(define top-center-right
  (list (make-image "i-top-teal-right.png")
	(make-image "top-teal-right.png")))

;; 2x20
(define top-center-right-space
  (list (make-image "i-top-teal-right.png")
	(make-image "top-teal-right.png")))

;; top-border-left
(define left-top-border
  (list (make-image "i-left-top-sting.png")
	(make-image "left-top-sting.png")))
(define left-top-border-shaped
  (list (make-image "i-shade-left.png")
	(make-image "shade-left.png")))

;; top-sting-left
(define left-top-sting
  (list (make-image "i-left-bottom-sting.png")
	(make-image "left-bottom-sting.png")))

;; 5x16*
(define left-border
  (list (make-image "i-teal-border-right.png")
	(make-image "teal-border-right.png")))

;; 21x63
(define right-top-border
  (list (make-image "i-right-top-icons.png")
	(make-image "right-top-icons.png")))
(define right-top-border-shaped
  (list (make-image "i-shade-right.png")
	(make-image "shade-right.png")))

;; 21x53
(define right-top-sting
  (list (make-image "i-right-end.png")
	(make-image "right-end.png")))
(define right-top-sting-shaped
  (list (make-image "i-right-end.png")
	(make-image "right-end.png")))

;; 6x16*
(define right-border
  (list (make-image "i-teal-border-right.png")
	(make-image "teal-border-right.png")))

;; 6x33
(define right-bottom-sting
  (list (make-image "i-right-bottom-sting.png")
	(make-image "right-bottom-sting.png")))

;; 6x29
(define right-bottom-corner
  (list (make-image "i-right-bottom-edge.png")
	(make-image "right-bottom-edge.png")))

;; 4x4
(define bottom-left-corner
  (list (make-image "i-bottom-left-corner.png")
	(make-image "bottom-left-corner.png")))

;; 
(define bottom-border
  (list (make-image "i-bottom-teal.png")
	(make-image "bottom-teal.png")))

;; 32x6
(define bottom-right-sting
  (list (make-image "i-bottom-right-sting.png")
	(make-image "bottom-right-sting.png")))

;; 6x6
(define bottom-right-corner
  (list (make-image "i-bottom-right-edge.png")
	(make-image "bottom-right-edge.png")))

;; BUTTONs

;; EXIT-Button
(define button-exit-images (list 
    (make-image "a-b-exit.png")
    (make-image "n-b-exit.png")
    (make-image "o-b-exit.png")
    (make-image "n-b-exit.png")
    ))

;; MAX-Button
(define button-max-images (list 
    (make-image "a-b-max.png")
    (make-image "n-b-max.png")
    (make-image "o-b-max.png")
    (make-image "n-b-max.png")
    ))
    
;; SHAPE-Button
(define button-shape-images (list 
    (make-image "a-b-shade.png")
    (make-image "n-b-shade.png")
    (make-image "o-b-shade.png")
    (make-image "n-b-shade.png")
    ))

;; SHAPE-SHAPE-Button
(define button-shape-images-shaped (list 
    (make-image "a-b-shade.png")
    (make-image "n-b-shade.png")
    (make-image "o-b-shade.png")
    (make-image "n-b-shade.png")
    ))

;; ICON-Button
(define button-icon-images (list 
    (make-image "o-b-icon.png")
    (make-image "n-b-icon.png")
    (make-image "a-b-icon.png")
    (make-image "o-b-icon.png")
    ))
    
;; BIG-Button
(define button-big-images (list
    (make-image "i-b-big.png")
    (make-image "n-b-big.png")
    (make-image "a-b-big.png")
    (make-image "i-b-big.png")
    ))

#|
;; UP-Button
(define button-up-images (list
    (make-image "i-b-up.png")
    (make-image "n-b-up.png")
    (make-image "a-b-up.png")
    (make-image "i-b-up.png")
    ))
    
;; DOWN-Button
(define button-down-images (list
    (make-image "i-b-down.png")
    (make-image "n-b-down.png")
    (make-image "a-b-down.png")
    (make-image "i-b-down.png")
    ))
|#

;; MENU-Button
(define button-menu-images (list
    (make-image "i-b-menu.png")
    (make-image "n-b-menu.png")
    (make-image "a-b-menu.png")
    (make-image "i-b-menu.png")
    ))


;; geometry computations

(define (title-left-width w)
  (max 0 (min (- (car (window-dimensions w))
		(text-width (window-name w)) 16
		))))

(define (title-right-width w)
  (max 0 (min (- (car (window-dimensions w))
		(text-width (window-name w))
		))))
    
(define (vertical-justification)
  ;; `center' justification adjusted for the 3-pixel top-border
  (max 0 (- (/ (- 22 (font-height default-font)) 2) 2)))


;; frames

(define common-frame-parts
  `(((background . ,top-left-border)
     (left-edge . -21)
     (top-edge . -24)
     (class . top-left-corner))

    ((background . ,top-left)
     (left-edge . 0)
     (top-edge . -22)
     (class . top-border))
     
    ((background . ,top-center-left)
     (left-edge . 28)
     (width . ,title-left-width)
     (top-edge . -6)
     (class . title))
     
    ((background . ,top-center-mid)
     (left-edge . ,(lambda (w) (max 0 (- (title-left-width w) 36))))
     (top-edge . -20)
     (class . title))

    ((background . ,top-center-right-space)
     (left-edge . ,(lambda (w) (- (title-right-width w) 10)))
     (top-edge . -20)
     (width . 10)
     (class . title))
    
    ((background . ,top-center-right)
     (left-edge . ,(lambda (w) (- (title-right-width w) 0)))
     (text . ,window-name)
     (font . ("-adobe-helvetica-medium-r-normal-*-10-*-*-*-p-*-iso10646-1" "-adobe-helvetica-bold-r-normal-*-10-*-*-*-p-*-iso10646-1" "-adobe-helvetica-bold-r-normal-*-10-*-*-*-p-*-iso10646-1" "-adobe-helvetica-bold-r-normal-*-10-*-*-*-p-*-iso10646-1"))
     (foreground . ("grey10" "black"))
     (x-justify . ,text-justifier)
     (y-justify . ,vertical-justification)
     (right-edge . 0)
     (top-edge . -20)
     (class . title))

    ((background . ,top-right-border)
     (right-edge . -21)
     (top-edge . -18)
     (class . top-right-corner))

    ((background . ,button-exit-images)
     (right-edge . -20)
     (top-edge . -19)
     (class . close-button))
#|
    ((background . ,button-down-images)
     (left-edge . 12)
     (top-edge . -20)
     (class . raise-window-depth))
|#
    ((background . ,button-big-images)
     (left-edge . -20)
     (top-edge . -23)
     (class . title))
#|
    ((background . ,button-up-images)
     (left-edge . -1)
     (top-edge . -17)
     (class . lower-window-depth))
|#     
))

(define normal-frame
  `(,@common-frame-parts

    ((background . ,right-top-border)
     (right-edge . -21)
     (top-edge . 0)
     (class . top-right-corner))

    ((background . ,right-top-sting)
     (right-edge . -21)
     (top-edge . 63)
     (class . right-border))

    ((background . ,right-bottom-sting)
     (right-edge . -6)
     (bottom-edge . 29)
     (class . right-border))

    ((background . ,right-bottom-corner)
     (right-edge . -6)
     (bottom-edge . 0)
     (class . bottom-right-corner))

    ((background . ,left-top-border)
     (left-edge . -21)
     (top-edge . 0)
     (class . left-border))

    ((background . ,left-top-sting)
     (left-edge . -6)
     (top-edge . 55)
     (class . left-border))

    ((background . ,left-border)
     (left-edge . -4)
     (top-edge . 82)
     (bottom-edge . 0)
     (class . left-border))

    ((background . ,right-border)
     (right-edge . -4)
     (top-edge . 116)
     (bottom-edge . 62)
     (class . right-border))

    ((background . ,bottom-left-corner)
     (left-edge . -4)
     (bottom-edge . -4)
     (class . bottom-left-corner))

    ((background . ,bottom-border)
     (left-edge . 0)
     (right-edge . 47)
     (bottom-edge . -4)
     (class . bottom-border))

    ((background . ,bottom-right-sting)
     (right-edge . 24)
     (bottom-edge . -6)
     (class . bottom-right-corner))

    ((background . ,bottom-right-corner)
     (right-edge . -6)
     (bottom-edge . -6)
     (class . bottom-right-corner))

    ((background . ,button-max-images)
     (right-edge . -16)
     (top-edge . 1)
     (class . maximize-button))
     
    ((background . ,button-shape-images)
     (right-edge . -16)
     (top-edge . 26)
     (class . shade-button))
     
    ((background . ,button-icon-images)
     (right-edge . -16)
     (top-edge . 49)
     (class . iconify-button))

    ((background . ,button-menu-images)
     (left-edge . -18)
     (top-edge . -4)
     (class . menu-button))

))

(define shaped-frame
  `(,@common-frame-parts

    ((background . ,right-top-border-shaped)
     (right-edge . -24)
     (top-edge . -6)
     (class . top-right-corner))

    ((background . ,left-top-border-shaped)
     (left-edge . -23)
     (top-edge . -6)
     (class . top-left-corner))

    ((background . ,button-big-images)
     (left-edge . -20)
     (top-edge . -23)
     (class . title))
))     



;; misc stuff

(define (rebuild-all)
  (rebuild-frames-with-style 'aBOX))

(define (reframe-all)
  (reframe-windows-with-style 'aBOX))

(define (color-changed)
  (recolor-all)
  (reframe-all))


(define initialize-gtkrc
  (let ((done nil))
    (lambda ()
      (unless done
	(require 'gtkrc)
	;; recolour everything when the GTK theme changes
	(gtkrc-call-after-changed color-changed)
	(setq done t)))))

;; setup the initial colours

(add-frame-style 'aBOX
    (lambda (w type)
	(case type
	    ((default) normal-frame)
	    ((transient) normal-frame)
	    ((shaped) shaped-frame)
	    ((shaped-transient) shaped-frame)
	)))
	
(call-after-property-changed
     'WM_NAME (lambda ()
           (rebuild-frames-with-style 'aBOX))))